// Copyright (c) 2001, Jan Hermanns & Arno Haase
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the
// following conditions are met:
//
//   * Redistributions of source code must retain the above
//     copyright notice, this list of conditions and the
//     following disclaimer.
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the
//     following disclaimer in the documentation and/or other
//     materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package safejdbc;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.ArrayList;


/**
 * An Implementation of the ResultSetIterator interface. This
 * convenience implementation iterates through the entire ResultSet
 * and calls the method forEachRow for each row (pun intended :)
 * ). While that is not the only reasonable way to handle a ResultSet,
 * it is common enough to warrant special support as provided by this
 * class.
 */
public abstract class ResultSetIterator implements ResultSetProcessor {

    private static final Class[] RESULTSET_CLASS = new Class[]{ResultSet.class};
    private static final ClassLoader _classLoader = ResultSetIterator.class.getClassLoader ();


    /**
     * This template method is called for each row of the ResultSet.
     * Implement this method to specify how the ResultSet is to be
     * handled.<br/>
     *
     * CAUTION: The methods "close" and "next" MUST not be called on
     * the ResultSet in this method. Iteration and closing of the
     * ResultSet is done by the framework!
     */
    public abstract void forEachRow (ResultSet rs) throws SQLException;


    /**
     * This method is called by the framework. This method iterates through the
     * ResultSet and calls forEachRow for each row (pun intended).
     */
    public final void process (ResultSet rs) throws SQLException {
        final InvocationHandler handler =
            new ProtectedResultSetHandler (rs);
        final ResultSet protectedResultSet =
            (ResultSet) Proxy.newProxyInstance (_classLoader, RESULTSET_CLASS, handler);

        // since protectedResultSet holds a reference to rs, this call to "next"
        // affects protectedResultSet as well but bypasses the checks there
        while (rs.next ()) {
            forEachRow (protectedResultSet);
        }
    }


    /**
     *  This class uses dynamic proxy functionality to prevent calls
     *  to "next" or "close" on a ResultSet. A dynamic proxy was used
     *  instead of a "conventional" Decorator pattern because
     *  ResultSet has very many methods and this is shorter.  It also
     *  has better testability.
     */
    private static class ProtectedResultSetHandler
        implements InvocationHandler {
        
        private final ResultSet _inner;
        private final Collection _forbiddenMethodNames = new ArrayList ();

        public ProtectedResultSetHandler (ResultSet inner) {
            _inner = inner;

            _forbiddenMethodNames.add ("next");
            _forbiddenMethodNames.add ("close");
        }

        public Object invoke (Object proxy, Method method, Object[] args)
                throws Throwable {
            if (_forbiddenMethodNames.contains (method.getName())) {
                throw new UnsupportedOperationException ("'next' and 'close' are called by the framework");
            }

            try {
                return method.invoke (_inner, args);
            }
            catch (IllegalAccessException e) {
                throw new InternalError (e.toString ());
            }
            catch (IllegalArgumentException e) {
                throw new InternalError (e.toString ());
            }
            catch (InvocationTargetException e) {
                throw e.getTargetException ();
            }
        }
    }
}
