// Copyright (c) 2001, Jan Hermanns & Arno Haase
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the
// following conditions are met:
//
//   * Redistributions of source code must retain the above
//     copyright notice, this list of conditions and the
//     following disclaimer.
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the
//     following disclaimer in the documentation and/or other
//     materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package safejdbc;


import java.sql.SQLException;


/**
 * This is the central abstraction of the JDBC library, it provides
 * methods to execute SQL statements. It is implemented by
 * <a href="SQLExecuter.html">SQLExecuter</a> to provide non-transactional
 * access to the database. Transactional access to the database is provided
 * through a callback mechanism inside the executeTX method of SQLExecuter which
 * passes another implementation of this interface to the TXCodeBlock.<br/>
 * For details see the <a href="package-summary.html>summary</a> and the
 * documentation of <a href="TXCodeBlock.html">TXCodeBlock</a>.<br/>
 */
public interface SQL {

    /**
     * This method is a convenience method for executing a SQL query
     * which has a single row with a single column containing a
     * number as a result.  This method is useful for queries such as
     * "SELECT COUNT(*) FROM ... WHERE ...".<br>
     *
     * <b>Example:</b>
     * <pre>
     * int nr_customers = sql.singleIntQuery("SELECT COUNT(*) FROM customer");
     * </pre>
     *
     * @param sqlStr contains the SQL query
     * @return the number in the result set (i.e. the number of matches)
     */
    public int singleIntQuery (String sqlStmt) throws SQLException;


    /**
     * This method is a convenience method for executing a SQL query
     * which has a single row with a single column containing a
     * number as a result.  This method is useful for queries such as
     * "SELECT COUNT(*) FROM ... WHERE ...". <br>
     *
     * <b>Example:</b>
     * <pre>
     * final int id_threshold = 1;
     *       
     * int some_customers =
     *     sql.singleIntQuery("SELECT COUNT(*) FROM customer WHERE id > ?",
     *                        new FillingCommand() {
     *                            public void fill(PreparedStatement s) throws SQLException {
     *                                s.setInt(1, id_threshold);
     *                            }});
     * </pre>
     *
     * @param sqlStr contains the SQL query
     * @param cmd is called to set the IN parameters of the prepared statement
     * @return the number in the result set (i.e. the number of matches)
     */
    public int singleIntQuery (String prepSqlStmt, FillingCommand cmd) throws SQLException;

    /**
     * Use this method to execute SQL-Query-Statements such as
     * <i>select from</i>.<br>
     *
     * <b>Example:</b>
     * <pre>
     * sql.query("SELECT id, name FROM customer WHERE id > 0",
     *           new ResultSetIterator() {
     *               public void forEachRow(ResultSet rs) throws SQLException {
     *                   System.out.println(rs.getString("name"));
     *               }});
     * </pre>
     *
     * @param sqlStr contains the SQL query     
     * @param rsProcessor is used to process the ResultSet resulting
     *   from the query. The ResultSet of this SQL-Query will be
     *   handed to the ResultSetProcessor; if an iteration over all
     *   rows of the result set is desired, use a ResultSetIterator
     *   (like in the above example).
     */
    public void query (String sqlStr, ResultSetProcessor rsProcessor)
            throws SQLException;


    /**
     * Use this method to execute modifying SQL-statements such as
     * <i>insert into</i> or <i>update</i>.<br>
     *
     * <b>Example:</b>
     * <pre>
     * sql.update("INSERT INTO customer (id, name) VALUES (1, 'John Smith')");
     * </pre>
     *
     * @param sqlStr the statement to be executed
     * @return the number of altered rows.
     */
    public int update (String sqlStr) throws SQLException;

    /**
     * Use this method to execute modifying SQL-statements such as
     * <i>insert into</i> or <i>update</i>, with a
     * <i>PreparedStatement</i>. The statement is executed in a separate
     * transaction, the change is committed immediately.<br>
     *
     * <b>Example:</b>
     * <pre>
     * final int id = 1;
     * final String name = "John Smith";
     *  
     * sql.update("INSERT INTO customer (id, name) VALUES (?, ?)",
     *            new FillingCommand() {
     *                public void fill(PreparedStatement s) throws SQLException {
     *                    s.setInt(1, id);
     *                    s.setString(2, name);
     *                }});
     * </pre>
     *
     * @param sqlStr the prepared statement to be executed
     * @return the number of altered rows.
     */
    public int update (String prepSql, FillingCommand cmd)
            throws SQLException;


    /**
     * Use this method to execute SQL-Query-Statements such as
     * <i>select from</i>. <br>
     *
     *
     * <b>Example:</b>
     * <pre>
     * final int id_threshold = 3;
     *      
     * sql.query("SELECT id, name FROM customer WHERE id > ?",
     *           new FillingCommand() {
     *               public void fill(PreparedStatement s) throws SQLException {
     *                   s.setInt(1, id_threshold);
     *               }},
     *           new ResultSetIterator() {
     *               public void forEachRow(ResultSet rs) throws SQLException {
     *                   System.out.println(rs.getString("name"));
     *               }});
     * </pre>
     *
     * @param prepSql contains the SQL query in the "PreparedStatement" form
     * @param cmd is called to set the IN parameters of the prepared statement
     * @param rsProcessor is used to process the ResultSet resulting from the query.
     * The ResultSet of this SQL-Query will be handed to the ResultSetProcessor; if
     * an iteration over all rows of the result set is desired, use a ResultSetIterator.
     */
    public void query
            (String prepSql, FillingCommand cmd, ResultSetProcessor rsProcessor)
            throws SQLException;
}
