// Copyright (c) 2001, Jan Hermanns & Arno Haase
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the
// following conditions are met:
//
//   * Redistributions of source code must retain the above
//     copyright notice, this list of conditions and the
//     following disclaimer.
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the
//     following disclaimer in the documentation and/or other
//     materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package safejdbc;

import safejdbc.holder.IntHolder;
import safejdbc.holder.BooleanHolder;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.util.Map;

/**
 * This is probably the most important class within the SafeJDBC
 * framework, because this class provides methods to actually access
 * the database.
 */
public class SQLExecuter implements SQL {


// --------------------------- a t t r i b u t e s ---------------------------


    private ConnectionProvider _conprovider = null;


// -------------------------- c o n s t r u c t o r --------------------------

    /**
     * To create an SQLExecuter instance a ConnectionProvider is
     * needed. Every method that accesses the database, addresses at
     * first the ConnectionProvider to get a connection. So be sure to
     * use a well designed ConnectionProvider implementation. <b>A bad
     * ConnectionProvider implementation is definitely a perfomance
     * penalty for your application.</b>
     */
    public SQLExecuter (ConnectionProvider conprovider) {
        _conprovider = conprovider;
    }


// ---------------- p u b l i c   m e t h o d s -----------------------------


    /**
     * Use this method, to execute several database calls in a single
     * transaction. If an exception occurs the transaction is rolled
     * back.<br>
     *     
     * <b>Example 1:</b>     
     * <pre>
     * sqlexecuter.executeTX(new TXCodeBlock() {
     *     public void doTransaction (SQL sql) throws SQLException {
     *         sql.update ("INSERT INTO customer (id, name) VALUES (1, 'Arno Haase')");
     *         sql.update ("INSERT INTO customer (id, name) VALUES (2, 'Jan Hermanns')");
     *     }});
     * </pre>
     *
     * Note that the SQL instance passed to doTransaction() can
     * also be handed to methods outside the TXCodeBlock. This is
     * extremly usefull to factor out code.<br>     
     *
     * <b>Example 2:</b>This example does the same as the above.
     * <pre>
     * private void insertArno(SQL sql) throws SQLException {
     *     sql.update ("INSERT INTO customer (id, name) VALUES (1, 'Arno Haase')");
     * }
     *
     * private void insertJan(SQL sql) throws SQLException {
     *     sql.update ("INSERT INTO customer (id, name) VALUES (2, 'Jan Hermanns')");
     * }
     *
     * sqlexecuter.executeTX(new TXCodeBlock() {
     *     public void doTransaction (SQL sql) throws SQLException {
     *         insertArno(sql);
     *         insertJan(sql);
     *     }});
     * </pre>
     *
     *
     * @param txCodeBlock contains a bunch of statements, to be
     * executed in a single transaction.  */
    public void executeTX (TXCodeBlock txCodeBlock) throws SQLException {

        SQLExceptionHelper helper = new SQLExceptionHelper ();
        RuntimeException rtExc = null;
        Error err = null;
        Connection txCon = null;

        try {
            txCon = _conprovider.getConnection ();
            txCon.setAutoCommit (false);
            txCodeBlock.doTransaction (new TXSQL (txCon));
            txCon.commit ();
        }
        catch (RuntimeException x) {
            rtExc = x;
            txCon.rollback ();
        }
        catch (SQLException x) {            // only in this case,
            helper.sqlExceptionOccurred (x); // we are interested in
            helper.rollback (txCon);         // tracing following SQLExceptions
        }
        catch (Error x) {
            err = x;
            txCon.rollback ();
        }

        finally {
            try {
                txCon.close ();
            }
            catch (SQLException x) {
                helper.sqlExceptionOccurred (x);
            }
            finally {
                if (rtExc != null)
                    throw rtExc;
                if (err != null)
                    throw err;
                // it is important, that this is the last check that
                // is performed.
                helper.processSQLException ();
            }
        }
    }


    public int update (final String sqlStr) throws SQLException {

        final IntHolder alteredRows = new IntHolder (0);

        executeTX (new TXCodeBlock () {
            public void doTransaction (SQL sql) throws SQLException {
                alteredRows.setValue (sql.update (sqlStr));
            }
        });

        return alteredRows.getValue ();
    }


    public int update (final String prepSql, final FillingCommand cmd)
            throws SQLException {

        final IntHolder alteredRows = new IntHolder (0);

        executeTX (new TXCodeBlock () {
            public void doTransaction (SQL sql) throws SQLException {
                alteredRows.setValue (sql.update (prepSql, cmd));
            }
        });

        return alteredRows.getValue ();
    }


    public void query
            (final String sqlStr, final ResultSetProcessor rsProcessor)
            throws SQLException {

        executeTX (new TXCodeBlock () {
            public void doTransaction (SQL sql) throws SQLException {
                sql.query (sqlStr, rsProcessor);
            }
        });
    }


    public void query (final String prepSql,
                       final FillingCommand cmd,
                       final ResultSetProcessor rsProcessor)
            throws SQLException {

        executeTX (new TXCodeBlock () {
            public void doTransaction (SQL sql) throws SQLException {
                sql.query (prepSql, cmd, rsProcessor);
            }
        });
    }


    /**
     *  This method is a convenience method for executing a SQL query
     *  which has a single row with a single column containing a
     *  number as a result.  This method is useful for queries such as
     *  "SELECT COUNT(*) FROM ... WHERE ...".<br/>
     *
     *  If the result set is empty or contains more than one row, an
     *  IllegalStateException is thrown.
     *
     *  @return the number in the result set
     */
    public int singleIntQuery (String sqlStmt) throws SQLException {
        final IntHolder resultHolder = new IntHolder (0);
        final BooleanHolder hasResult = new BooleanHolder (false);

        query (sqlStmt, new ResultSetIterator () {
            public  void forEachRow (ResultSet rs) throws SQLException {
                if (hasResult.getValue ()) {
                    throw new IllegalStateException("ResultSet has several rows");
                }
                hasResult.setValue(true);
                resultHolder.setValue (rs.getInt(1));
            }
        });
        if (! hasResult.getValue ()) {
            throw new IllegalStateException ("ResultSet was empty");
        }
        return resultHolder.getValue ();
    }


    /**
     *  This method is a convenience method for executing a SQL query
     *  which has a single row with a single column containing a
     *  number as a result.  This method is useful for queries such as
     *  "SELECT COUNT(*) FROM ... WHERE ...".<br/>
     *
     *  If the result set is empty or contains more than one row, an
     *  IllegalStateException is thrown.
     *
     *  @return the number in the result set
     */
    public int singleIntQuery (String prepSqlStmt, FillingCommand cmd)
        throws SQLException
    {    
        final IntHolder resultHolder = new IntHolder (0);
        final BooleanHolder hasResult = new BooleanHolder (false);

        query (prepSqlStmt, cmd, new ResultSetIterator () {
            public  void forEachRow (ResultSet rs) throws SQLException {
                if (hasResult.getValue ()) {
                    throw new IllegalStateException("ResultSet has several rows");
                }
                hasResult.setValue(true);
                resultHolder.setValue (rs.getInt(1));
            }
        });
        if (! hasResult.getValue ()) {
            throw new IllegalStateException ("ResultSet was empty");
        }
        return resultHolder.getValue ();
    }
}


