// Copyright (c) 2001, Jan Hermanns & Arno Haase
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the
// following conditions are met:
//
//   * Redistributions of source code must retain the above
//     copyright notice, this list of conditions and the
//     following disclaimer.
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the
//     following disclaimer in the documentation and/or other
//     materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package safejdbc;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.SQLException;
import java.util.Properties;


/**
 * This class provides JDBC connection via the JDBC 1 Driver.
 * But <b>this is only an example-implementation and should not be used in
 * real-world-projects</b>, because connections are not cached. This means
 * every call to getConnection() is delegated to the underlying
 * driver. The driver than creates a connection, wich is a very
 * expensive (time consuming) operation.
 */
public class SimpleConnectionProvider implements ConnectionProvider {

    private Driver _driver;
    private String _url;
    private Properties _info;


    /**
     * This constructor initializes the driver via separate strings.
     * <br>
     * <b>Example:</b>
     * <pre>
     * ConnectionProvider conprovider = new SimpleConnectionProvider
     *       ("hermanns", "password",
     *        "jdbc:db2://192.168.11.10:8400/Test",
     *        "COM.ibm.db2.jdbc.net.DB2Driver");     
     * </pre>
     *
     *  @param user specifies the database user that is used for login
     *  @param password specifies the database password that is used for login
     *  @param url specifies the database URL
     *  @param drivername contains the fully qualified class name of the driver class
     */
    public SimpleConnectionProvider
            (String user, String password, String url, String drivername)
            throws Exception {

        Properties info = new Properties ();

        info.put ("user", user);
        info.put ("password", password);

        _driver = (Driver) Class.forName (drivername).newInstance ();
        _url = url;
        _info = info;
    }


    /**
     *  This constructor initializes the driver using a map containing login information.
     *  @param url contains the database URL
     *  @param info must contain entries at least for the two keys "user" and "password",
     *    containing login information for the database
     *  @param drivername must contain the fully qualified class name of the driver class
     */
    public SimpleConnectionProvider (String url, Properties info, String drivername)
            throws Exception {

        _driver = (Driver) Class.forName (drivername).newInstance ();
        _url = url;
        _info = info;

    }


    /**
     *  This method is called by the framework to yield a database connection.
     */
    public Connection getConnection () throws SQLException {
        return _driver.connect (_url, _info);
    }

}
