// Copyright (c) 2001, Jan Hermanns & Arno Haase
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the
// following conditions are met:
//
//   * Redistributions of source code must retain the above
//     copyright notice, this list of conditions and the
//     following disclaimer.
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the
//     following disclaimer in the documentation and/or other
//     materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package safejdbc;

import safejdbc.holder.BooleanHolder;
import safejdbc.holder.IntHolder;

import java.sql.*;


//***** TODO: add documentation

/**
 * This class is only for internal use, to provide a SQL interface,
 * that can be used in a transactional context.
 */
class TXSQL implements SQL {


// --------------------------- a t t r i b u t e s ---------------------------


    private Connection _connection;


// -------------------------- c o n s t r u c t o r --------------------------


    TXSQL (Connection connection) {
        _connection = connection;
    }


// ---------------- p u b l i c   m e t h o d s -----------------------------



    public int update (String sqlStr) throws SQLException {
        SQLExceptionHelper helper = new SQLExceptionHelper ();
        int result = 0;
        Statement stmt = null;

        try {
            stmt = _connection.createStatement ();
            result = stmt.executeUpdate (sqlStr);
        }
        catch (SQLException x) {
            helper.sqlExceptionOccurred (x);
        }
        finally {
            helper.close (stmt);
            helper.processSQLException ();

            return result;
        }
    }


    public void query (String sqlStr, ResultSetProcessor rsProcessor)
            throws SQLException {

        SQLExceptionHelper helper = new SQLExceptionHelper ();
        Statement stmt = null;
        ResultSet rs = null;

        try {
            stmt = _connection.createStatement ();
            rs = stmt.executeQuery (sqlStr);

            rsProcessor.process (rs);
        }
        catch (SQLException x) {
            helper.sqlExceptionOccurred (x);
        }
        finally {
            helper.close (rs, stmt);
            helper.processSQLException ();
        }
    }


    public int update (String prepSql, FillingCommand cmd)
            throws SQLException {

        SQLExceptionHelper helper = new SQLExceptionHelper ();
        int result = 0;
        PreparedStatement stmt = null;

        try {
            stmt = _connection.prepareStatement (prepSql);

            cmd.fill (stmt);

            result = stmt.executeUpdate ();
        }
        catch (SQLException x) {
            helper.sqlExceptionOccurred (x);
        }
        finally {
            helper.close (stmt);
            helper.processSQLException ();

            return result;
        }
    }


    public void query
            (String prepSql, FillingCommand cmd, ResultSetProcessor rsProcessor)
            throws SQLException {

        SQLExceptionHelper helper = new SQLExceptionHelper ();
        PreparedStatement stmt = null;
        ResultSet rs = null;

        try {
            stmt = _connection.prepareStatement (prepSql);

            cmd.fill (stmt);

            rs = stmt.executeQuery ();

            rsProcessor.process (rs);
        }
        catch (SQLException x) {
            helper.sqlExceptionOccurred (x);
        }
        finally {
            helper.close (rs, stmt);
            helper.processSQLException ();
        }

    }


    /**
     *  This method is a convenience method for executing a SQL query which
     *  has a single row with a single column containing a number as a result.
     *  This method is useful for queries such as "SELECT COUNT(*) FROM ... WHERE ...".
     */
    public int singleIntQuery (String sqlStmt) throws SQLException {
        final IntHolder resultHolder = new IntHolder (0);
        final BooleanHolder hasResult = new BooleanHolder (false);

        query (sqlStmt, new ResultSetIterator () {
            public void forEachRow (ResultSet rs) throws SQLException {
                if (hasResult.getValue ()) {
                    throw new IllegalStateException ("ResultSet has several rows");
                }
                hasResult.setValue (true);
                resultHolder.setValue (rs.getInt (1));
            }
        });
        if (!hasResult.getValue ()) {
            throw new IllegalStateException ("ResultSet was empty");
        }
        return resultHolder.getValue ();
    }


    /**
     *  This method is a convenience method for executing a SQL query which
     *  has a single row with a single column containing a number as a result.
     *  This method is useful for queries such as "SELECT COUNT(*) FROM ... WHERE ...".
     */
    public int singleIntQuery (String prepSqlStmt, FillingCommand cmd) throws SQLException {
        final IntHolder resultHolder = new IntHolder (0);
        final BooleanHolder hasResult = new BooleanHolder (false);

        query (prepSqlStmt, cmd, new ResultSetIterator () {
            public void forEachRow (ResultSet rs) throws SQLException {
                if (hasResult.getValue ()) {
                    throw new IllegalStateException ("ResultSet has several rows");
                }
                hasResult.setValue (true);
                resultHolder.setValue (rs.getInt (1));
            }
        });
        if (!hasResult.getValue ()) {
            throw new IllegalStateException ("ResultSet was empty");
        }
        return resultHolder.getValue ();
    }
}
